<?php

/**
 * @file
 * Functions need to work with import/export files from Desk-Net.
 */

/**
 * Perform upload file from Desk-Net.
 *
 * @param string $filename
 *   The filename.
 * @param string $desk_net_file
 *   The string containing the contents of the file.
 *
 * @return array
 *   The data about saving file.
 */
function desk_net_import_files($filename, $desk_net_file) {
  $sanitize_filename = desk_net_sanitize_file_name($filename);
  $basic_filename = preg_replace('/\.[^.]+$/', '', $sanitize_filename);

  $local_file_fid = desk_net_get_file_by_name($basic_filename);

  if ($local_file_fid !== FALSE && !empty($local_file_fid->filename)) {
    // Compare local- and import-file.
    $file_id = desk_net_compare_file_hash($local_file_fid, $sanitize_filename);

    if ($file_id !== FALSE) {
      return desk_net_generate_response($file_id);
    }
  }
  $save_file = file_save_data($desk_net_file, 'public://' . $sanitize_filename);

  if ($save_file != FALSE) {
    // Generate response.
    $response_data = desk_net_generate_response($save_file->fid);
  }
  else {
    $response_data = desk_net_generate_response(NULL);
  }

  return $response_data;
}

/**
 * Perform attache file to Article.
 *
 * @param array $data
 *   The post data.
 * @param object $node
 *   The node data.
 *
 * @return string
 *   The content with html code.
 */
function desk_net_attache_file_to_article(array $data, $node) {
  // Create empty content.
  $html_content = '';
  // Check file_id in Desk-Net data.
  foreach ($data['tasks'] as $attache_content) {
    if (!empty($attache_content['files'])) {
      foreach ($attache_content['files'] as $file) {
        if (isset($file['cmsId']) && !desk_net_scan_content_on_dn_files($node, $file['cmsId'])) {
          // Update file_name.
          $basic_filename = desk_net_sanitize_file_name($file['filename']);
          $local_file = file_load($file['cmsId']);

          if ($local_file !== FALSE && $local_file->filename == $basic_filename) {
            // Check content settings.
            $file_action = desk_net_get_content_settings($local_file->type);
            // Attache file to node.
            if ($file_action !== 'only-media') {
              // Insert file id.
              $html_content .= desk_net_generate_html_content($file['cmsId'], $data['id']);
            }
          }
        }
      }
    }
    elseif (!empty($attache_content['content']) && !desk_net_scan_content_on_dn_files($node, $attache_content['id'])) {
      $html_content .= desk_net_generate_html_content(NULL, $attache_content['id'], $attache_content['content']);
    }
  }

  return $html_content;
}

/**
 * Performing compare import file with file saved in Drupal.
 *
 * @param object $local_file
 *   The local file.
 * @param string $desk_net_file
 *   The Desk-Net filename.
 *
 * @return int|bool
 *   The result compare.
 */
function desk_net_compare_file_hash($local_file, $desk_net_file) {
  if (md5($local_file->filename) === md5($desk_net_file)) {
    return $local_file->fid;
  }

  return FALSE;
}

/**
 * Performing compare import file with file saved in Drupal.
 *
 * @param string $basic_filename
 *   The name import-file.
 *
 * @return array|bool
 *   The file.
 */
function desk_net_get_file_by_name($basic_filename) {
  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'file')
    ->propertyCondition('filename', "$basic_filename%", 'LIKE');
  $result = $query->execute();

  if (!empty($result['file'])) {
    $file_id = array_keys($result['file']);

    $result = file_load($file_id[0]);
  }
  else {
    $result = FALSE;
  }

  return $result;
}

/**
 * Performing generate response for request.
 *
 * @param int $file_id
 *   The file id was save.
 *
 * @return array
 *   The response on request.
 */
function desk_net_generate_response($file_id) {
  $response_data = array(
    'id' => $file_id,
    'cmsEditLink' => url("file/$file_id/edit", array('absolute' => TRUE)),
    'cmsOpenLink' => file_create_url($file_id),
  );

  return $response_data;
}

/**
 * Performing get value content settings.
 *
 * @param int $file_type
 *   The will file type attache.
 *
 * @return array
 *   The update action.
 */
function desk_net_get_content_settings($file_type) {
  switch ($file_type) {
    case 'image':
      $value = desk_net_variable_get('desk_net_content_settings_picture-files');
      break;

    case 'video':
      $value = desk_net_variable_get('desk_net_content_settings_video-files');
      break;

    default:
      $value = desk_net_variable_get('desk_net_content_settings_other-files');
  }
  // Check user settings on empty value and set default.
  if ($value === NULL) {
    $value = 'body-media';
  }

  return $value;
}

/**
 * Performing generate short_code.
 *
 * @param int $file_id
 *   The file id.
 * @param int $story_id
 *   The story id.
 * @param string $content
 *   The content message.
 *
 * @return string
 *   The content with html code.
 */
function desk_net_generate_html_content($file_id = NULL, $story_id = NULL, $content = NULL) {
  if ($file_id !== NULL) {
    $file = file_load($file_id);
    // Didn't find file.
    if ($file === FALSE) {
      return '';
    }
    $file_url = file_create_url($file->uri);
    $content_template = $file->type;
    $basic_filename = preg_replace('/\.[^.]+$/', '', $file->filename);
  }
  else {
    $content_template = 'content';
  }
  // Generate Html code.
  switch ($content_template) {
    case 'video':
      $short_code = "<p><video class='dn-file-id-$file_id' width=\"480\" height=\"360\" controls>";
      $short_code .= "<source src=\"$file_url\" type=\"$file->filemime\">";
      $short_code .= "Your browser does not support the video tag.";
      $short_code .= "</video></p>";
      break;

    case 'audio':
      $short_code = "<p><audio class='dn-file-id-$file_id' controls>";
      $short_code .= "<source src=\"$file_url\" type=\"$file->filemime\">";
      $short_code .= "Your browser does not support the audio element.";
      $short_code .= "</audio></p>";
      break;

    case 'image':
      $image = image_get_info($file->uri);
      $content = array(
        'file' => array(
          '#theme' => 'image',
          '#attributes' => array(
            'class' => "dn-file-id-$file_id",
          ),
          '#alt' => "$basic_filename",
          '#path' => file_create_url($file->uri),
          '#width' => $image['width'],
          '#height' => $image['height'],
        ),
      );
      $short_code = drupal_render($content);
      $short_code = "<p>$short_code</p>";
      break;

    case 'content':
      $short_code = "<p class=\"dn-file-id-$story_id\">$content</p>";
      break;

    default:
      $short_code = "<p><a class=\"dn-file-id-$file_id\" href=\"$file_url\">$basic_filename</a></p>";
  }

  return $short_code;
}

/**
 * The scan article content on files/text from Desk-Net.
 *
 * @param object $node
 *   The article.
 * @param int $file_id
 *   The new file id.
 *
 * @return bool
 *   The result scan files.
 */
function desk_net_scan_content_on_dn_files($node, $file_id) {
  if (!empty($node->body[LANGUAGE_NONE][0]['value'])) {
    $search_result = preg_match("/\\bdn\\-file\\-id\\-$file_id+[\\'|\\\"|\\:|\\s]/",
      $node->body[LANGUAGE_NONE][0]['value'], $old_content);
    if (!$search_result) {
      return FALSE;
    }
    else {
      return TRUE;
    }
  }
  else {
    return FALSE;
  }
}

/**
 * Check content changes by user in Drupal.
 *
 * @param object $node
 *   The node data.
 */
function desk_net_check_content_changed_by_user($node) {
  $old_content = variable_get("desk-net-node-$node->vid-content");
  variable_del("desk-net-node-$node->vid-content");
  // Empty content.
  $new_value = '';

  if (isset($node->body[LANGUAGE_NONE][0]['value'])) {
    $new_value = $node->body[LANGUAGE_NONE][0]['value'];
  }
  $new_content = md5($new_value);

  if ($old_content !== $new_content) {
    $node->desk_net_content_change_status[LANGUAGE_NONE][0]['value'] = 'changed';
  }
}
