<?php

/**
 * @file
 * Functions needed for create requests to Desk-Net API.
 */

include_once drupal_get_path('module', 'desk_net') . '/desk_net.module';

/**
 * Perform HTTP GET request.
 *
 * @param string $url
 *   Base API url.
 * @param string $type
 *   The Desk-Net API method.
 * @param string $record_id
 *   ID element.
 *
 * @return string
 *   The request result.
 */
function desk_net_get($url, $type, $record_id = '') {
  $request_url = $url . "/api/v1_0_1/{$type}/{$record_id}";
  $args = array(
    'headers' => [
      'Authorization' => 'bearer ' . desk_net_variable_get('desk_net_token'),
    ],
    'method' => 'GET',
  );

  $response = desk_net_send_request($request_url, $args);

  return $response;
}

/**
 * Perform HTTP PUT request.
 *
 * @param string $url
 *   Base API url.
 * @param int $record_id
 *   The ID element.
 * @param string $update_data
 *   The Data for send.
 *
 * @return string
 *   The request result.
 */
function desk_net_put($url, $record_id, $update_data) {
  $request_url = $url . "/api/v1_0_1/elements/{$record_id}";
  $args = array(
    'headers' => [
      'Authorization' => 'bearer ' . desk_net_variable_get('desk_net_token'),
      'Content-Type' => 'application/json;charset=UTF-8',
    ],
    'data' => $update_data,
    'method' => 'PUT',
  );

  $response = desk_net_send_request($request_url, $args);

  return $response;
}

/**
 * Perform HTTP POST/DELETE request.
 *
 * @param array $data
 *   The upload data.
 * @param string $url
 *   Base API url.
 * @param string $type
 *   The Desk-Net API method.
 * @param string $http_request
 *   Custom HTTP request.
 * @param string $record_id
 *   The ID element.
 *
 * @return string
 *   The additional information about story from Desk-Net.
 */
function desk_net_custom_request(array $data, $url, $type, $http_request, $record_id = '') {
  $request_url = $url . "/api/v1_0_1/{$type}/{$record_id}";

  if (!empty($data)) {
    $data = json_encode($data, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES);

    // Delete ampersand encode.
    $data = str_replace('&amp;', '&', $data);

    $args = array(
      'headers' => [
        'Authorization' => 'bearer ' . desk_net_variable_get('desk_net_token'),
        'Content-Length' => strlen($data),
        'Content-Type' => 'application/json;charset=UTF-8',
      ],
      'data' => $data,
    );
  }
  else {
    $args = array(
      'headers' => [
        'Authorization' => 'bearer ' . desk_net_variable_get('desk_net_token'),
        'Content-Type' => 'application/json;charset=UTF-8',
      ],
    );
  }

  $args['method'] = $http_request;

  $response = desk_net_send_request($request_url, $args, $data);

  return $response;
}

/**
 * Perform send HTTP Request.
 *
 * @param string $request_url
 *   The link for request.
 * @param array $args
 *   The params for request.
 * @param string $data
 *   The post data.
 * @param bool $last_request
 *   The attempt send request.
 *
 * @return string
 *   The request result.
 */
function desk_net_send_request($request_url, array $args, $data = NULL, $last_request = NULL) {
  $obj_response = drupal_http_request($request_url, $args);
  $http_code = $obj_response->code;

  switch ($http_code) {
    case 401:
      $response = desk_net_unauthorized($request_url, $args, $data, $last_request);
      break;

    case 400:
      if (empty($obj_response->data)) {
        return FALSE;
      }

      $response = desk_net_check_platform_schedule($obj_response);
      break;

    case 200:
      $response = $obj_response->data;
      break;

    default:
      if (empty($obj_response->data)) {
        return FALSE;
      }

      $data = json_decode($data, TRUE);
      $body_response = json_decode($obj_response->data, TRUE);
      $response = desk_net_check_error_message($body_response, $data['publications'][0]['cms_id']);
  }

  return $response;
}

/**
 * Perform check user authorized.
 *
 * @param string $request_url
 *   The link for request.
 * @param array $args
 *   The params for request.
 * @param string $data
 *   The post data.
 * @param bool $last_request
 *   The attempt send request.
 *
 * @return string|array
 *   The result getting new token.
 */
function desk_net_unauthorized($request_url, array $args, $data, $last_request) {
  $token = desk_net_get_token(desk_net_variable_get('desk_net_login'), desk_net_variable_get('desk_net_password'));

  if ($token !== FALSE) {
    desk_net_variable_set('desk_net_token', $token);
  }

  $response = 'desk_net_unauthorized';

  if ($last_request != 'update_token' && $token !== FALSE) {
    $args['headers']['Authorization'] = "bearer $token";
    $response = desk_net_send_request($request_url, $args, $data, 'update_token');
  }

  return $response;
}

/**
 * Perform check platform schedule in response from request.
 *
 * @param object $response
 *   The request response.
 *
 * @return string|array
 *   The request mark.
 */
function desk_net_check_platform_schedule($response) {
  $body_response = json_decode($response->data, TRUE);

  if (preg_match("/^Publication date doesn't match platform schedule/", $body_response['message'])) {
    $response = 'platform_schedule';
  }
  return $response;
}

/**
 * Perform GET TOKEN.
 *
 * @param string $login
 *   The Desk-Net user login.
 * @param string $password
 *   The Desk-Net user password.
 *
 * @return string
 *   The result getting token from Desk-Net.
 */
function desk_net_get_token($login, $password) {
  // The request params.
  $args = array(
    'headers' => [
      'Content-Type' => 'application/x-www-form-urlencoded',
    ],
    'data' => http_build_query([
      'grant_type' => 'client_credentials',
      'client_id' => $login,
      'client_secret' => $password,
    ]),
    'method' => 'POST',
  );

  // The HTTP POST request for get Token.
  $obj_response = drupal_http_request(DN_BASE_URL . '/api/token', $args);

  if ($obj_response->code != 200) {
    desk_net_variable_set('desk_net_token', 'not_valid');

    return FALSE;
  }

  // Update token option.
  $response_data = json_decode($obj_response->data, TRUE);
  desk_net_variable_set('desk_net_token', $response_data['access_token']);

  return $response_data['access_token'];
}

/**
 * Perform check request Error Message from Desk-Net.
 *
 * @param array $message_content
 *   The list message.
 * @param string $node_id
 *   The node ID in Drupal.
 *
 * @return bool
 *   The result of scanning - error message.
 */
function desk_net_check_error_message(array $message_content, $node_id) {
  if (preg_match('/^Page with id \[\d.*\] was not found/', $message_content['message'])) {
    desk_net_get_category();
    $node = node_load($node_id);
    // Second attempt for node sending.
    desk_net_create_node_drupal_to_dn($node);
  }
  elseif (preg_match('/^User with id \[\d.*\] was not found/', $message_content['message'])) {
    preg_match('/\d.*\d/', $message_content['message'], $desk_net_user_id);
    $node = node_load($node_id);
    $user = user_load($node->uid);

    if (isset($user->desk_net_author_id) && !empty($user->desk_net_author_id[LANGUAGE_NONE][0]['value'])) {
      $edit = array(
        'desk_net_author_id' => array(
          LANGUAGE_NONE => array(
            0 => array(
              'value' => NULL,
            ),
          ),
        ),
      );
      user_save($user, $edit);
    }
    else {
      drupal_set_message(desk_net_module_message(4), 'error');

      return 'not_show_new_notice';
    }

    desk_net_create_node_drupal_to_dn($node);
  }
  else {
    drupal_set_message(desk_net_module_message(4), 'error');

    return 'not_show_new_notice';
  }

  return 'update_module_data';
}
